@@ -15,7 +15,6 @@
 import datetime
 import re
 
-
 from django.conf import settings
 from django.db import models
 from django.utils.encoding import smart_unicode
@@ -41,15 +40,51 @@ def timestamp2datetime(timestamp, timezone=None):
     return datetime.datetime.fromtimestamp(timestamp, tzinfo)
 
 
+
+class PhpBBReverseSingleRelatedObjectDescriptor(models.fields.related.ReverseSingleRelatedObjectDescriptor):
+    """
+    Same as original, but accepts 0 also as none related objects.
+    """
+    def __get__(self, instance, instance_type=None):
+        if instance is None:
+            return self
+
+        try:
+            return getattr(instance, self.cache_name)
+        except AttributeError:
+            val = getattr(instance, self.field.attname)
+            if val in (0, None, ""):
+                return None
+
+        return super(PhpBBReverseSingleRelatedObjectDescriptor, self).__get__(instance, instance_type)
+
+
 class PhpBBForeignKey(models.ForeignKey):
     """
     phpBB stores a None ForeignKey as a numeric 0
     """
+    def __init__(self, *args, **kwargs):
+        kwargs2 = {
+            "blank": True, "null": True,
+            "default": 0,
+        }
+        kwargs2.update(kwargs)
+        super(PhpBBForeignKey, self).__init__(*args, **kwargs2)
+
+    def to_python(self, value):
+        if value in ("", None, 0):
+            return None
+        return super(PhpBBForeignKey, self).to_python(value)
+
     def get_db_prep_save(self, value, connection):
-        if value in ("", None):
-            value = 0
+        if value in ("", None, 0):
+            return None
         return super(PhpBBForeignKey, self).get_db_prep_save(value, connection)
 
+    def contribute_to_class(self, cls, name):
+        super(PhpBBForeignKey, self).contribute_to_class(cls, name)
+        setattr(cls, self.name, PhpBBReverseSingleRelatedObjectDescriptor(self))
+
 #------------------------------------------------------------------------------
 # important models first:
 
@@ -77,9 +112,8 @@ class User(models.Model):
         help_text="A cached copy of the user's computed permissions."
     )
     # user_perm_from = models.ForeignKey(, db_column="# user_perm_from"
-    perm_from = PhpBBForeignKey("User", db_column="user_perm_from", blank=True, null=True,
-        # mediumint(8) unsigned
-        default=0,
+    perm_from = PhpBBForeignKey("User", db_column="user_perm_from",
+        # mediumint(8) unsigned - default=0,
         help_text="The id of the user whose permissions are being tested. {{fk|users|user_id}}"
     )
     ip = models.CharField(max_length=40, db_column="user_ip", blank=True,
@@ -122,7 +156,7 @@ class User(models.Model):
         default=0,
         help_text="A hash of the user's email address."
     )
-    birthday = models.CharField(max_length=10, db_column="user_birthday",
+    birthday = models.CharField(max_length=10, db_column="user_birthday", blank=True,
         # varchar(10)
         help_text="The user's birthday, in the form of dd-mm-yyyy."
     )
@@ -141,7 +175,7 @@ class User(models.Model):
         default=0,
         help_text="The time of the latest post of the user, UNIX timestamp"
     )
-    lastpage = models.CharField(max_length=200, db_column="user_lastpage",
+    lastpage = models.CharField(max_length=200, db_column="user_lastpage", blank=True,
         # varchar(200)
         help_text="The last page visited by the user."
     )
@@ -210,12 +244,11 @@ class User(models.Model):
         help_text="Style the user uses to browse the board. {{fk|styles|style_id}}"
     )
     # user_rank = models.IntegerField()
-    rank = models.ForeignKey("Rank", db_column="user_rank", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    rank = PhpBBForeignKey("Rank", db_column="user_rank",
+        # mediumint(8) unsigned - default=0,
         help_text="User's rank. {{fk|ranks|rank_id}}"
     )
-    colour = models.CharField(max_length=6, db_column="user_colour",
+    colour = models.CharField(max_length=6, db_column="user_colour", blank=True,
         # varchar(6)
         help_text="User's colour, hex code."
     )
@@ -399,12 +432,37 @@ class User(models.Model):
     reminded = models.IntegerField(db_column="user_reminded",)
     reminded_time = models.IntegerField(db_column="user_reminded_time",)
 
+    def has_content(self):
+        """
+        Manually check if user has a post or topic
+        """
+        if Topic.objects.all().filter(poster=self).count():
+            return True
+        if Post.objects.all().filter(poster=self).count():
+            return True
+        if Attachment.objects.all().filter(poster=self).count():
+            return True
+        return False
+
     def registration_datetime(self):
         return timestamp2datetime(self.regdate, self.timezone)
 
     def lastvisit_datetime(self):
         return timestamp2datetime(self.lastvisit, self.timezone)
 
+    def get_cleaned_signature(self):
+        """
+        returns a cleaned version of the self.sig bbcode.
+        """
+        if self.sig_bbcode_bitfield:
+            bbcode_uid = self.sig_bbcode_uid
+        else:
+            bbcode_uid = None
+
+        text = smart_unicode(self.sig)
+        bbcode = clean_bbcode(text, bbcode_uid)
+        return bbcode
+
     def __unicode__(self):
         return smart_unicode(self.username)
 
@@ -605,9 +663,8 @@ class Post(models.Model):
         # mediumint(8) unsigned
         default=0,
     )
-    poster = models.ForeignKey("User", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    poster = PhpBBForeignKey("User",
+        # mediumint(8) unsigned - default=0,
     )
     icon = models.ForeignKey("Icon", blank=True,
         # mediumint(8) unsigned
@@ -659,7 +716,7 @@ class Post(models.Model):
     attachment = models.PositiveSmallIntegerField(db_column="post_attachment",
         # tinyint(1) unsigned
         default=0,
-        help_text="| 1=This post has at least one attachment&lt;br/>0=no attachments in this post"
+        help_text="1=This post has at least one attachment 0=no attachments in this post"
     )
     bbcode_bitfield = models.CharField(max_length=255,
         # varchar(255)
@@ -743,15 +800,13 @@ class Topic(models.Model):
         help_text="Primary key"
     )
     # forum_id = models.IntegerField()
-    forum = models.ForeignKey("Forum", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    forum = PhpBBForeignKey("Forum",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|forums|forum_id}}"
     )
     # icon_id = models.IntegerField()
-    icon = models.ForeignKey("Icon", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    icon = PhpBBForeignKey("Icon",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|icons|icon_id}}"
     )
     attachment = models.PositiveSmallIntegerField(db_column="topic_attachment",
@@ -774,9 +829,8 @@ class Topic(models.Model):
         help_text="The title of the topic."
     )
     # topic_poster = models.IntegerField()
-    poster = models.ForeignKey("User", db_column="topic_poster", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    poster = PhpBBForeignKey("User", db_column="topic_poster",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|users|user_id}}"
     )
     time = models.PositiveIntegerField(db_column="topic_time",
@@ -815,9 +869,8 @@ class Topic(models.Model):
         help_text="[[Constants|POST_NORMAL]](0), POST_STICKY(1), POST_ANNOUNCE(2) or POST_GLOBAL(3)"
     )
     # topic_first_post_id = models.IntegerField()
-    first_post = models.ForeignKey("Post", related_name='+', blank=True, db_column="topic_first_post_id",
-        # mediumint(8) unsigned
-        default=0,
+    first_post = PhpBBForeignKey("Post", related_name='+', db_column="topic_first_post_id",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|posts|post_id}}"
     )
     first_poster_name = models.CharField(max_length=255, db_column="topic_first_poster_name",
@@ -829,15 +882,13 @@ class Topic(models.Model):
         help_text="The colour of the topic creator's default user group."
     )
     # topic_last_post_id = models.IntegerField()
-    last_post = models.ForeignKey("Post", related_name='+', blank=True, db_column="topic_last_post_id",
-        # mediumint(8) unsigned
-        default=0,
+    last_post = PhpBBForeignKey("Post", related_name='+', db_column="topic_last_post_id",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|posts|post_id}}"
     )
     # topic_last_poster_id = models.IntegerField()
-    last_poster = models.ForeignKey("User", related_name='+', blank=True, db_column="topic_last_poster_id",
-        # mediumint(8) unsigned
-        default=0,
+    last_poster = PhpBBForeignKey("User", related_name='+', db_column="topic_last_poster_id",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|users|user_id}}"
     )
     last_poster_name = models.CharField(max_length=255, db_column="topic_last_poster_name",
@@ -873,9 +924,8 @@ class Topic(models.Model):
         help_text="Has this topic been bumped? 1 (yes), 0(no)"
     )
     # topic_bumper = models.IntegerField()
-    bumper = models.ForeignKey("User", related_name='+', db_column="topic_bumper", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    bumper = PhpBBForeignKey("User", related_name='+', db_column="topic_bumper",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|users|user_id}}"
     )
     poll_title = models.CharField(max_length=255,
@@ -1044,9 +1094,8 @@ class Attachment(models.Model):
         help_text="1 if attachment is used inside private message, 0 if used inside post"
     )
     # poster_id = models.IntegerField()
-    poster = models.ForeignKey("User", blank=True,
-        # mediumint(8) unsigned
-        default=0,
+    poster = PhpBBForeignKey("User",
+        # mediumint(8) unsigned - default=0,
         help_text="{{fk|users|user_id}}"
     )
     is_orphan = models.PositiveSmallIntegerField(
